#
# Copyright 2001 Stfane Fermigier and Nuxeo SARL
# See LICENSE.TXT for licensing information
#
"NuxDocument for the CMF."

from zLOG import LOG, DEBUG, ERROR
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
#from Products.CMFCore.WorkflowCore import WorkflowAction, afterCreate
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.utils import bodyfinder

from NuxDocument import NuxDocument

ADD_CONTENT_PERMISSION = 'Add portal content'

factory_type_information = ({
  'id': 'NuxDocument',
  'meta_type': 'CMFNuxDocument',
  'description': """Generic documents.""",
  'icon': 'document_icon.gif',
  'product': 'NuxDocument',
  'factory': 'addCMFNuxDocument',
  'immediate_view': 'metadata_edit_form',
  'actions': (
    {
      'id': 'download',
      'name': 'Download',
      'action': '',
      'permissions': (CMFCorePermissions.View,)
    }, {
      'id': 'view',
      'name': 'View',
      'action': 'NuxDocument_view',
      'permissions': (CMFCorePermissions.View,)
    }, { 
      'id': 'edit',
      'name': 'Edit',
      'action': 'NuxDocument_edit_form',
      'permissions': (CMFCorePermissions.ModifyPortalContent,)
    }, {
      'id': 'metadata',
      'name': 'Metadata',
      'action': 'metadata_edit_form',
      'permissions': (CMFCorePermissions.ModifyPortalContent,)
    })
},)


def addCMFNuxDocument(self, id, title='', file=''):
    """Add a NuxDocument"""
    o = CMFNuxDocument(id, title=title, file=file)
    self._setObject(id, o)


class CMFNuxDocument(PortalContent, NuxDocument, DefaultDublinCoreImpl):
    "A Document - Handles whatever type is understood by the plugins"

    meta_type = 'CMFNuxDocument'
    effective_date = expiration_date = None
    _isDiscussable = 1
    isPrincipiaFolderish = 0
    __dav_collection__ = 0
    isAnObjectManager = 0

    security = ClassSecurityInfo()

    def __init__(self, id, title='', file=''):
        DefaultDublinCoreImpl.__init__(self, title=title)
        NuxDocument.__init__(self, id=id, file=file)

    def _edit(self, title=None, file=None, mime_type=None):
        NuxDocument._edit(self, title, file, mime_type)
        self.reindexObject() # from PortalContent

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    edit = _edit

    security.declareProtected('View', 'index_html')
    def index_html(self, RESPONSE=None):
        'Default view'
        return self.document_src(RESPONSE=RESPONSE)

    security.declareProtected('View', 'CookedBody')
    def CookedBody(self):
        return bodyfinder(self.getHtml())

    security.declareProtected('View', 'SearchableText')
    def SearchableText(self):
        "Get content suitable for Cataloging"
        return "%s %s %s" % (
            self.title, self.Description(), self.PrincipiaSearchSource())

    def _get_icon_name(self):
        # XXX the plugins should provide the icon, refactor later
        mt = self.MimeType()
        icons = {
            'application/pdf':               'icon_pdf.gif',
            'text/msword':                   'icon_doc.gif',
            'application/msword':            'icon_doc.gif',
            'application/vnd.ms-msword':     'icon_doc.gif',
            'application/vnd.ms-powerpoint': 'icon_ppt.gif',
            'application/vnd.ms-excel':      'icon_xls.gif',
        }
        return icons.get(mt, 'document_icon.gif')

    security.declareProtected('View', 'getIcon')
    def getIcon(self, relative_to_portal=0):
        """Get the document icon."""
        icon = self._get_icon_name()
        if relative_to_portal:
            return icon
        portal_url = getToolByName(self, 'portal_url')
        res = portal_url(relative=1) + '/' + icon
        while res[:1] == '/':
            res = res[1:]
        return res

    security.declarePublic('icon')
    icon = getIcon # For the ZMI

    def manage_beforeDelete(self, item, container):
        LOG('CMFNuxDocument', DEBUG, 'manage_beforeDelete')
        PortalContent.manage_beforeDelete(self, item, container)
        NuxDocument.manage_beforeDelete(self, item, container)

    def manage_afterClone(self, item):
        LOG('CMFNuxDocument', DEBUG, 'manage_afterClone')
        PortalContent.manage_afterClone(self, item)
        NuxDocument.manage_afterClone(self, item)

    def manage_afterAdd(self, item, container):
        LOG('CMFNuxDocument', DEBUG, 'manage_afterAdd')
        PortalContent.manage_afterAdd(self, item, container)
        NuxDocument.manage_afterAdd(self, item, container)

    def PUT_handler(self, data, mime_type):
        """Handles WebDAVs PUT requests for CPS"""
        self._edit(file=data, mime_type=mime_type)


    security.declareProtected('Edit NuxDocument', 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests"""
        LOG('CMFNuxDocument', DEBUG, 'PUT')
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        mime_type = REQUEST.get_header('content-type', None)
        file=REQUEST['BODYFILE']
        self._edit(file=file, mime_type=mime_type)
        self.reindexObject()

        RESPONSE.setStatus(204)
        return RESPONSE


InitializeClass(CMFNuxDocument)
